;!function(){try{var k="_btDebugIds",u="undefined",v="161b73a2-f2a9-75db-db52-57c86d1aef80",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../lib/errors");
const backtrace_service_1 = require("backtrace-service");
const git_routines_1 = require("../lib/git_routines");
const git_routines_2 = require("../lib/git_routines");
const util_1 = require("../lib/util");
const fs_1 = __importDefault(require("fs"));
const rimraf_1 = __importDefault(require("rimraf"));
const log = backtrace_service_1.Logger.getLogger();
const viewableColumns = [
    "id",
    "name",
    "url",
    "default_commit",
    "project_name",
    "universe_name",
    "position",
    "auth_type",
    "clone_status",
    "clone_error",
    "created_at",
    "updated_at",
    "autoclone_submodules",
    "last_fetched",
    "archive_secret",
];
const repositoriesController = (db, service, urlPrefix) => {
    service.app.get(`${urlPrefix}/:universeName/:projectName/repositories`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.asyncWrapper)(async (req, res, _next) => {
        const { projectName, universeName } = req.params;
        let repositories;
        repositories = await db("repositories")
            .select(viewableColumns)
            .where({ project_name: projectName, universe_name: universeName });
        return res.send({ repositories });
    }));
    service.app.put(`${urlPrefix}/:universeName/:projectName/repositories/:id`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.adminCheck)(service), (0, util_1.asyncWrapper)(async (req, res, _next) => {
        const id = req.params.id;
        const repo = await db("repositories").first("*").where("id", id);
        if (!repo) {
            return res
                .status(404)
                .send((0, errors_1.getError)(5, `Repository with id ${id} not found`));
        }
        const updateProps = {};
        const { name, url, default_commit, auth_type, user_name, password, ssh_public_key, ssh_private_key, commit_rules, autoclone_submodules, } = req.body;
        if (name && name.trim() !== "") {
            updateProps.name = name.trim();
        }
        if (url && url.trim() !== "") {
            updateProps.url = url.trim();
        }
        if (default_commit && default_commit.trim() !== "") {
            updateProps.default_commit = default_commit.trim();
        }
        if (auth_type && auth_type.trim() !== "") {
            updateProps.auth_type = auth_type.trim();
        }
        if (user_name && user_name.trim() !== "") {
            updateProps.user_name = user_name.trim();
        }
        if (password && password.trim() !== "") {
            updateProps.password = password.trim();
        }
        if (ssh_public_key && ssh_public_key.trim() !== "") {
            updateProps.ssh_public_key = ssh_public_key.trim();
        }
        if (ssh_private_key && ssh_private_key.trim() !== "") {
            updateProps.ssh_private_key = ssh_private_key.trim();
        }
        if (autoclone_submodules !== null &&
            autoclone_submodules !== undefined) {
            updateProps.autoclone_submodules = autoclone_submodules;
        }
        if (commit_rules &&
            Array.isArray(commit_rules) &&
            commit_rules.length > 0) {
            const trimmed = commit_rules
                .map((cr) => cr.trim())
                .filter((cr) => cr.length > 0);
            if (trimmed.length > 0) {
                await db("commit_rules").where({ repository_id: id }).del();
                trimmed.forEach(async (val, idx) => await db("commit_rules").insert({
                    repository_id: id,
                    position: idx,
                    attribute_template: val,
                }));
            }
        }
        await db("repositories")
            .first("id")
            .where("id", id)
            .update(updateProps);
        const updatedRepoAllColumns = await db("repositories")
            .first("*")
            .where("id", id);
        if (updatedRepoAllColumns.clone_status === "complete" &&
            updatedRepoAllColumns.url !== repo.url) {
            const pathToRepo = (0, git_routines_1.repoPath)(updatedRepoAllColumns.id);
            const gitRepo = await git_routines_2.newGit.getRepo(updatedRepoAllColumns.id);
            await git_routines_2.newGit.addOrUpdateRemote(pathToRepo, "origin", updatedRepoAllColumns.url);
            await (0, git_routines_2.gitFetch)(gitRepo, updatedRepoAllColumns, db);
        }
        if (updatedRepoAllColumns.clone_status === "error" &&
            (updatedRepoAllColumns.url !== repo.url ||
                updatedRepoAllColumns.auth_type !== repo.auth_type ||
                updatedRepoAllColumns.user_name !== repo.auth_name ||
                updatedRepoAllColumns.password !== repo.password ||
                updatedRepoAllColumns.ssh_public_key !== repo.ssh_public_key ||
                updatedRepoAllColumns.ssh_private_key !== repo.ssh_private_key)) {
            try {
                const clonePath = (0, git_routines_1.repoPath)(id);
                if (fs_1.default.existsSync(clonePath)) {
                    rimraf_1.default.sync(clonePath);
                }
                await (0, git_routines_2.gitCloneExn)(updatedRepoAllColumns);
                await db("repositories")
                    .select("id")
                    .where("id", id)
                    .update({ clone_status: "complete", clone_error: null });
            }
            catch (e) {
                const msg = (0, util_1.toFriendlyErrMsg)(e.message);
                await db("repositories")
                    .select("id")
                    .where("id", id)
                    .update({ clone_error: msg });
            }
        }
        const updatedRepoViewableColumns = await db("repositories")
            .first(viewableColumns)
            .where("id", id);
        res.json({ repository: updatedRepoViewableColumns });
    }));
    service.app.post(`${urlPrefix}/:universeName/:projectName/repositories`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.adminCheck)(service), (0, util_1.asyncWrapper)(async (req, res, next) => {
        const { auth_type, project_name, universe_name, autoclone_submodules, } = req.body;
        let { name, url, default_commit, user_name, password, ssh_public_key, ssh_private_key, } = req.body;
        name = name === null || name === void 0 ? void 0 : name.trim();
        url = url === null || url === void 0 ? void 0 : url.trim();
        default_commit = default_commit === null || default_commit === void 0 ? void 0 : default_commit.trim();
        user_name = user_name === null || user_name === void 0 ? void 0 : user_name.trim();
        password = password === null || password === void 0 ? void 0 : password.trim();
        ssh_public_key = ssh_public_key === null || ssh_public_key === void 0 ? void 0 : ssh_public_key.trim();
        ssh_private_key = ssh_private_key === null || ssh_private_key === void 0 ? void 0 : ssh_private_key.trim();
        const [{ maxPos }] = await db("repositories")
            .max("position", { as: "maxPos" })
            .where("universe_name", universe_name)
            .where("project_name", project_name);
        const position = Number.isFinite(maxPos) ? maxPos + 1 : 0;
        let id;
        try {
            [id] = await db("repositories").insert({
                name,
                url,
                default_commit,
                auth_type,
                user_name,
                password,
                ssh_public_key,
                ssh_private_key,
                project_name,
                universe_name,
                clone_status: "in_progress",
                position,
                coroner_location: req.headers["X-Coroner-Location"],
                autoclone_submodules,
            });
            const repo = await db("repositories")
                .first(viewableColumns)
                .where("id", id);
            res.json({
                repository: repo,
            });
        }
        catch (e) {
            if (e.errno === 19 &&
                /UNIQUE constraint failed: repositories\.name/.test(e.message)) {
                return res
                    .status(400)
                    .send((0, errors_1.getError)(7, "UNIQUE constraint: repositories.name"));
            }
            return next(e);
        }
        if (!Number.isInteger(id)) {
            return;
        }
        try {
            const clonePath = (0, git_routines_1.repoPath)(id);
            if (fs_1.default.existsSync(clonePath)) {
                rimraf_1.default.sync(clonePath);
            }
            await (0, git_routines_2.gitCloneExn)({
                id,
                url,
                auth_type,
                user_name,
                password,
                ssh_private_key,
                ssh_public_key,
                universe_name,
                project_name,
                name,
            });
            await db("repositories")
                .select("id")
                .where("id", id)
                .update({ clone_status: "complete" });
        }
        catch (e) {
            const msg = (0, util_1.toFriendlyErrMsg)(e.message);
            await db("repositories")
                .select("id")
                .where("id", id)
                .update({ clone_status: "error", clone_error: msg });
        }
    }));
    service.app.delete(`${urlPrefix}/:universeName/:projectName/repositories/:id`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.adminCheck)(service), (0, util_1.asyncWrapper)(async (req, res, _next) => {
        const { id } = req.params;
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send((0, errors_1.getError)(2, "Invalid rule ID"));
        }
        const repo = await db("repositories").first("*").where("id", id);
        if (!repo) {
            return res.status(404).send();
        }
        await db("repositories").where("id", id).del();
        const rules = await db("rules")
            .select("id")
            .where("matching_repository_id", id);
        for (const rule of rules) {
            await db("rules").where("id", rule.id).del();
        }
        const commit_rules = await db("commit_rules")
            .select("id")
            .where("repository_id", id);
        for (const commit_rule of commit_rules) {
            await db("commit_rules").where("id", commit_rule.id).del();
        }
        const pathToRepo = (0, git_routines_1.repoPath)(repo.id);
        rimraf_1.default.sync(pathToRepo);
        return res.status(200).send();
    }));
    service.app.get(`${urlPrefix}/:universeName/:projectName/repositories/:id/fetch`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.asyncWrapper)(async (req, res, _next) => {
        const { id } = req.params;
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send((0, errors_1.getError)(2, "Invalid ID"));
        }
        const repo = await db("repositories").first("*").where("id", id);
        if (!repo) {
            return res.status(404).send();
        }
        try {
            if (repo.clone_status === "error") {
                try {
                    const clonePath = (0, git_routines_1.repoPath)(id);
                    if (fs_1.default.existsSync(clonePath)) {
                        rimraf_1.default.sync(clonePath);
                    }
                    await (0, git_routines_2.gitCloneExn)(repo);
                    await db("repositories")
                        .select("id")
                        .where("id", id)
                        .update({ clone_status: "complete", clone_error: null });
                }
                catch (e) {
                    const msg = (0, util_1.toFriendlyErrMsg)(e.message);
                    await db("repositories")
                        .select("id")
                        .where("id", id)
                        .update({ clone_error: msg });
                    throw e;
                }
            }
            const gitRepo = await git_routines_2.newGit.getRepo(repo.id);
            await (0, git_routines_2.gitFetch)(gitRepo, repo, db);
            return res.status(200).send({});
        }
        catch (e) {
            const msg = (0, util_1.toFriendlyErrMsg)(e.message);
            return res.status(400).send((0, errors_1.unknownError)(msg));
        }
    }));
    service.app.get(`${urlPrefix}/:universeName/:projectName/repositories/:id/commit/:commit/submodules`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.asyncWrapper)(async (req, res, _next) => {
        const { id, commit, universeName, projectName } = req.params;
        if (!commit) {
            return res.status(422).send((0, errors_1.getError)(10, "Invalid commit"));
        }
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send((0, errors_1.getError)(2, "Invalid ID"));
        }
        const repo = await db("repositories").first("*").where("id", id);
        if (!repo) {
            return res.status(404).send((0, errors_1.getError)(5));
        }
        try {
            const submoduleData = await (0, git_routines_1.readGitmodules)(repo.id, commit, universeName, projectName);
            return res.send({ submodules: submoduleData });
        }
        catch (e) {
            return res.send({ submodules: {} });
        }
    }));
    service.app.post(`${urlPrefix}/:universeName/:projectName/repositories/:id/commit/:commit/submodules/clone`, service.identityManager.authenticateRequest(), (0, util_1.accessCheck)(service), (0, util_1.adminCheck)(service), (0, util_1.asyncWrapper)(async (req, res, _next) => {
        const { id, commit, projectName, universeName } = req.params;
        if (!commit) {
            return res.status(422).send((0, errors_1.getError)(10, "Invalid commit"));
        }
        if (!Number.isInteger(parseInt(id, 10))) {
            return res.status(422).send((0, errors_1.getError)(2, "Invalid ID"));
        }
        const parentRepo = await db("repositories").first("*").where("id", id);
        if (!parentRepo) {
            return res.status(404).send((0, errors_1.getError)(5));
        }
        for (const subName in req.body.submodules) {
            const url = req.body.submodules[subName].url;
            const default_commit = req.body.submodules[subName].default_commit || "master";
            if (subName && url) {
                let hasAtSymbol = /@/.test(url);
                let sshPrefix = new RegExp("ssh://").test(url);
                let auth_type = null;
                let user_name = null;
                let password = null;
                let ssh_public_key = null;
                let ssh_private_key;
                if (hasAtSymbol || sshPrefix) {
                    auth_type = "ssh_key";
                    ssh_public_key = parentRepo.ssh_public_key;
                    ssh_private_key = parentRepo.ssh_private_key;
                }
                else {
                    auth_type = "none";
                }
                let newRepoId;
                try {
                    [newRepoId] = await db("repositories").insert({
                        name: subName,
                        url,
                        default_commit,
                        auth_type,
                        user_name,
                        password,
                        ssh_public_key,
                        ssh_private_key,
                        project_name: projectName,
                        universe_name: universeName,
                        clone_status: "in_progress",
                        submodule_parent_repo: parentRepo.id,
                        submodule_origin_commit: commit,
                        position: 0,
                        coroner_location: req.headers["X-Coroner-Location"],
                    });
                }
                catch (e) {
                    if (e.errno === 19 &&
                        /UNIQUE constraint failed: repositories\.name/.test(e.message)) {
                        log.warn("Unique repo name constraint error while cloning submodule " +
                            subName);
                    }
                    else {
                        log.warn("Error while cloning submodule " + subName);
                    }
                    continue;
                }
                try {
                    const clonePath = (0, git_routines_1.repoPath)(newRepoId);
                    if (fs_1.default.existsSync(clonePath)) {
                        rimraf_1.default.sync(clonePath);
                    }
                    const repo = await db("repositories").first("*").where("id", id);
                    await (0, git_routines_2.gitCloneExn)(repo);
                    await db("repositories")
                        .select("id")
                        .where("id", newRepoId)
                        .update({ clone_status: "complete" });
                }
                catch (e) {
                    const msg = (0, util_1.toFriendlyErrMsg)(e.message);
                    await db("repositories")
                        .select("id")
                        .where("id", newRepoId)
                        .update({ clone_status: "error", clone_error: msg });
                }
            }
        }
        res.status(200).json({});
    }));
};
exports.default = repositoriesController;
//# sourceMappingURL=repositories.js.map
//# debugId=161b73a2-f2a9-75db-db52-57c86d1aef80