;!function(){try{var k="_btDebugIds",u="undefined",v="8fd1046e-e9eb-7272-8594-1cc12024e5a7",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.p4Trust = exports.p4Available = exports.getNewTicket = exports.p4Annotate = exports.p4Describe = exports.p4Filelog = exports.p4ListFiles = void 0;
const child_process_1 = require("child_process");
const db_1 = __importDefault(require("./db"));
const backtrace_service_1 = require("backtrace-service");
const path_1 = __importDefault(require("path"));
const log = backtrace_service_1.Logger.getLogger();
const loginNotNecessaryRE = new RegExp("login.*? not .*necessary");
const pwInvalidRE = new RegExp("password .*?invalid");
const sessionExpiredRE = new RegExp("(?:session .*?expired)|(?:login .*?again)");
const ticketRe = new RegExp("^([A-Z0-9]+)$", "m");
const p4CmdPath = () => {
    let cachedPath = null;
    return (cachedPath ||
        (function p4CmdPath2() {
            var _a;
            if (!cachedPath) {
                cachedPath =
                    process.env.P4CMD ||
                        ((_a = global.service.configuration) === null || _a === void 0 ? void 0 : _a.p4cmd) ||
                        "p4";
            }
            return cachedPath;
        })());
};
const p4TrustPath = () => {
    let cachedPath = null;
    return (cachedPath ||
        (function p4TrustPath2() {
            var _a;
            if (!cachedPath) {
                cachedPath =
                    process.env.P4TRUST ||
                        ((_a = global.service.configuration) === null || _a === void 0 ? void 0 : _a.p4trust) ||
                        path_1.default.resolve(".p4trust");
            }
            return cachedPath;
        })());
};
async function getTicket(server) {
    var _a;
    let ticket = await ((_a = (0, db_1.default)("perforce_servers").first("*").where("id", server.id)) === null || _a === void 0 ? void 0 : _a.auth_ticket);
    if (!ticket) {
        ticket = await getNewTicket(server);
    }
    return ticket;
}
async function getNewTicket(server) {
    try {
        const newTicket = await p4Login(server);
        if ("id" in server) {
            await (0, db_1.default)("perforce_servers")
                .first("*")
                .where("id", server.id)
                .update("auth_ticket", newTicket);
        }
        return newTicket;
    }
    catch (e) {
        if (pwInvalidRE.test(e)) {
            if ("id" in server) {
                await (0, db_1.default)("perforce_servers").first("*").where("id", server.id).update({
                    error: e,
                    connection_status: "not_working",
                });
            }
            return null;
        }
        throw e;
    }
}
exports.getNewTicket = getNewTicket;
async function p4Trust(server, untrust = false) {
    const { host, port, user_name } = server;
    const flags = untrust ? "-d" : "-yf";
    if (/^ssl:/.test(host)) {
        const { stderr, error } = (0, child_process_1.spawnSync)(p4CmdPath(), ["trust", flags], {
            env: {
                P4USER: user_name,
                P4PORT: `${host}:${port}`,
                P4TRUST: p4TrustPath(),
            },
            timeout: 4000,
        });
        if (error) {
            throw error;
        }
        if (stderr.length > 0) {
            const msg = stderr.toString();
            throw new Error(msg);
        }
    }
}
exports.p4Trust = p4Trust;
async function p4Login(server) {
    return new Promise((resolve, reject) => {
        const { host, port, password, user_name } = server;
        const echo = (0, child_process_1.spawn)("echo", [password || ""]);
        const login = (0, child_process_1.spawn)(p4CmdPath(), ["login", "-p"], {
            env: {
                P4USER: user_name,
                P4PORT: `${host}:${port}`,
                P4TRUST: p4TrustPath(),
            },
        });
        setTimeout(function p4LoginTimeoutHandler() {
            const err = new Error("Connection timed out");
            err.code = "ETIMEDOUT";
            reject(err);
        }, 6500);
        echo.stdout.on("data", (data) => {
            login.stdin.write(data.toString());
        });
        echo.stderr.on("data", (data) => {
            reject(data.toString());
        });
        echo.on("close", (code) => {
            if (code !== 0) {
                reject();
            }
            login.stdin.end();
        });
        login.stdout.on("data", (data) => {
            const str = data.toString();
            const match = ticketRe.exec(str);
            if (match) {
                resolve(match[1]);
            }
            if (loginNotNecessaryRE.test(str)) {
                resolve("LOGIN_NOT_NECESSARY");
            }
        });
        login.stderr.on("data", (data) => {
            reject(data.toString());
        });
        login.on("close", (code) => {
            if (code !== 0) {
                reject();
            }
        });
    });
}
async function p4Cmd(args, server, alreadyRetried = false, timeout = 2000) {
    await p4Trust(server);
    const ticket = await getTicket(server);
    const { host, port, user_name } = server;
    const { stdout, stderr, error } = (0, child_process_1.spawnSync)(p4CmdPath(), ["-P", ticket, ...args], {
        env: {
            P4USER: user_name,
            P4PORT: `${host}:${port}`,
            P4TRUST: p4TrustPath(),
        },
        timeout,
    });
    if (error) {
        throw error;
    }
    if (stderr.length > 0) {
        const msg = stderr.toString();
        if (alreadyRetried) {
            throw new Error(msg);
        }
        if (pwInvalidRE.test(msg) || sessionExpiredRE.test(msg)) {
            await getNewTicket(server);
            return await p4Cmd(args, server, true);
        }
        throw new Error(msg);
    }
    return stdout.toString();
}
const filesRegex = /(\/\/.+)#(\d+)/;
async function p4ListFiles(fileName, server, depotPath, changelistId) {
    if (changelistId !== undefined) {
        fileName += "@" + changelistId;
    }
    let stdout;
    try {
        stdout = await p4Cmd(["files", "-e", depotPath + "..." + fileName], server, false, 10000);
    }
    catch (e) {
        if (/no such file/.test(e.message)) {
            return [];
        }
        else if (/must .*?refer .*?to .*?depot/.test(e.message)) {
            return [];
        }
        else {
            throw e;
        }
    }
    const lines = stdout.split("\n");
    const result = [];
    lines.forEach((line) => {
        const parsedLine = filesRegex.exec(line);
        if (parsedLine) {
            result.push([parsedLine[1], parsedLine[2]]);
        }
    });
    return result;
}
exports.p4ListFiles = p4ListFiles;
const filelogRegex = /change (\d+) .*? on (\d{4}\/\d{2}\/\d{2})/;
async function p4Filelog(filepath, revision, server) {
    const stdout = await p4Cmd(["filelog", `${filepath}#${revision},${revision}`], server);
    const lines = stdout.split("\n");
    const match = filelogRegex.exec(lines[1]);
    return match && { changelistId: match[1], changelistDate: match[2] };
}
exports.p4Filelog = p4Filelog;
async function p4Describe(changelistId, server) {
    return await p4Cmd(["describe", "-s", changelistId], server);
}
exports.p4Describe = p4Describe;
const annotateRe = /(\d+): (.+) (\d{4}\/\d{2}\/\d{2}) (.*)/;
async function p4Annotate(filepath, revision, server) {
    const stdout = await p4Cmd(["annotate", "-q", "-c", "-u", `${filepath}#${revision}`], server, false, 10000);
    const changelistIds = [];
    const blame = [];
    const timestamps = [];
    const fileContents = [];
    let totalLines = 0;
    stdout.split("\n").forEach((line) => {
        const parsedLine = annotateRe.exec(line);
        if (parsedLine) {
            totalLines += 1;
            changelistIds.push(parsedLine[1]);
            blame.push(parsedLine[2]);
            timestamps.push(parsedLine[3]);
            fileContents.push(parsedLine[4]);
        }
    });
    return {
        changelistIds,
        blame,
        timestamps,
        fileContents,
        totalLines,
    };
}
exports.p4Annotate = p4Annotate;
function p4Available() {
    const { stdout, stderr, error } = (0, child_process_1.spawnSync)(p4CmdPath(), ["-h"], {
        timeout: 1000,
    });
    if (error) {
        log.info("p4Available(): " + error.message);
        return false;
    }
    if (stderr.length > 0) {
        const msg = stderr.toString();
        if (/command not found/.test(msg)) {
            return false;
        }
        log.info("p4Available(): " + msg);
        return false;
    }
    if (/perforce/.test(stdout.toString())) {
        return true;
    }
    return false;
}
exports.p4Available = p4Available;
//# sourceMappingURL=perforce.js.map
//# debugId=8fd1046e-e9eb-7272-8594-1cc12024e5a7