;!function(){try{var k="_btDebugIds",u="undefined",v="0c21760d-641e-6a0c-c43b-257af6d0041a",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.killSSHAgent = exports.addPrivateKeyToAgent = exports.startSSHAgent = exports.getAgent = void 0;
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const pidFile = path_1.default.resolve(".ssh-agent.json");
function readAgentEnv() {
    if (fs_1.default.existsSync(pidFile)) {
        try {
            const pidFileContents = fs_1.default.readFileSync(pidFile, "utf8");
            const { SSH_AUTH_SOCK, SSH_AGENT_PID } = JSON.parse(pidFileContents);
            return { SSH_AUTH_SOCK, SSH_AGENT_PID };
        }
        catch (e) {
            console.error(`Error reading ${pidFile}: ${e.message}`);
            return null;
        }
    }
    else {
        return null;
    }
}
function writeAgentEnv(env) {
    try {
        fs_1.default.writeFileSync(pidFile, JSON.stringify(env));
    }
    catch (e) {
        console.error(`Error writing ${pidFile}: ${e.message}`);
        throw e;
    }
}
async function getAgent() {
    await ensureSSHAgent();
    const agentEnv = readAgentEnv();
    if (!agentEnv) {
        throw new Error("Error reading agent environment");
    }
    return Object.assign({}, agentEnv);
}
exports.getAgent = getAgent;
function agentIsActive() {
    const agentEnv = readAgentEnv();
    if (!agentEnv) {
        return false;
    }
    const pid = parseInt(agentEnv.SSH_AGENT_PID);
    if (pid <= 0 || isNaN(pid) || !Number.isInteger(pid) || pid === undefined) {
        return false;
    }
    try {
        process.kill(pid, 0);
        return true;
    }
    catch (e) {
        return false;
    }
}
const startSSHAgent = async () => {
    var _a, _b;
    const agent = (0, child_process_1.spawnSync)("ssh-agent");
    if (agent.error) {
        console.error(`Error starting ssh-agent: ${agent.error.message}`);
        return null;
    }
    const SSH_AUTH_SOCK = (_a = agent.stdout
        .toString()
        .match(/SSH_AUTH_SOCK=([^;]+)/)) === null || _a === void 0 ? void 0 : _a[1];
    const SSH_AGENT_PID = (_b = agent.stdout
        .toString()
        .match(/SSH_AGENT_PID=([^;]+)/)) === null || _b === void 0 ? void 0 : _b[1];
    if (!SSH_AUTH_SOCK || !SSH_AGENT_PID) {
        console.error("Error parsing ssh-agent output");
        return null;
    }
    let ssh_agent_env = {
        SSH_AUTH_SOCK,
        SSH_AGENT_PID,
    };
    writeAgentEnv(ssh_agent_env);
    return ssh_agent_env;
};
exports.startSSHAgent = startSSHAgent;
async function ensureSSHAgent() {
    if (!agentIsActive()) {
        const agent = await (0, exports.startSSHAgent)();
        if (!agent) {
            console.error("Error starting ssh-agent");
            return false;
        }
    }
    return true;
}
async function addPrivateKeyToAgent(privateKey) {
    const agentEnv = await getAgent();
    const addKey = (0, child_process_1.spawnSync)("ssh-add", ["-"], {
        env: agentEnv,
        input: privateKey,
    });
    if (addKey.status !== 0) {
        console.error(`Error adding key to ssh-agent (exit code ${addKey.status})`);
        console.error(addKey.stderr.toString(), addKey.stdout.toString());
        return false;
    }
    return true;
}
exports.addPrivateKeyToAgent = addPrivateKeyToAgent;
function killSSHAgent() {
    const SSH_AGENT_ENV = readAgentEnv();
    if (SSH_AGENT_ENV) {
        const killAgent = (0, child_process_1.spawnSync)("ssh-agent", ["-k"], {
            env: SSH_AGENT_ENV,
        });
        if (killAgent.status !== 0) {
            console.error(`Error killing ssh-agent (exit code ${killAgent.status})`);
        }
    }
    else {
        console.log("No ssh-agent to kill");
    }
}
exports.killSSHAgent = killSSHAgent;
//# sourceMappingURL=ssh.js.map
//# debugId=0c21760d-641e-6a0c-c43b-257af6d0041a