/// <reference types="node" />
import { BacktraceConfiguration } from '@backtrace/node';
import { IServiceDescriptor } from '..';
/**
 * Default service configuration
 */
export type IConfiguration<T> = T & {
    isProduction: boolean;
    descriptor?: IServiceDescriptor;
    metrics: IMetricsOptions & IRemoteLogger;
    logger: ILoggerOptions;
    backtrace: BacktraceConfiguration & IRemoteLogger & LegacyBacktraceNodeConfiguration;
    ssl: ISslOptions | undefined;
};
/**
 * Extend Backtrace-node/Circonus metrics configuration object with
 * properties that determine metrics/reporter behavior. Developer should use
 * these variables to determine if backtrace-service should use backtrace-node/circonus integration
 */
export interface IRemoteLogger {
    /**
     * Deprecated. Please use enable instead.
     */
    enabled?: boolean;
    enable?: boolean;
}
export interface LegacyBacktraceNodeConfiguration {
    endpoint?: string;
}
export interface ISslOptions {
    key: string | Buffer;
    cert: string | Buffer;
}
export interface ILoggerOptions {
    logLevel: string;
}
export interface IMetricsOptions {
    secret: string;
    uuid: string;
    interval?: number;
    hostUrl?: string;
    disableExpressMetrics?: boolean;
}
